@@ -8,6 +8,8 @@
 #include "gbfsservice.h"
 #include "gbfsstore.h"
 #include "gbfsjob.h"
+#include "gbfsreader.h"
+#include "gbfsvehicletypes.h"
 
 #include <KPublicTransport/Attribution>
 #include <KPublicTransport/Location>
@@ -15,14 +17,23 @@
 #include <KPublicTransport/LocationRequest>
 #include <KPublicTransport/RentalVehicle>
 
+#include <QCoreApplication>
 #include <QDebug>
 #include <QJsonArray>
 #include <QJsonDocument>
 #include <QJsonObject>
+#include <QTimer>
+
+#include <cmath>
+#include <functional>
 
 using namespace KPublicTransport;
 
-GBFSBackend::GBFSBackend() = default;
+GBFSBackend::GBFSBackend()
+{
+    QTimer::singleShot(std::chrono::seconds(10), Qt::CoarseTimer, QCoreApplication::instance(), []() { GBFSStore::expire(); });
+}
+
 GBFSBackend::~GBFSBackend() = default;
 
 AbstractBackend::Capabilities GBFSBackend::capabilities() const
@@ -57,24 +68,63 @@ static QString stationIdToString(const QJsonValue &id)
     return id.toString();
 }
 
+static RentalVehicle::VehicleType gbfs2kptVehicleType(const GBFSVehicleType &vehicle)
+{
+    static constexpr struct {
+        GBFSVehicleType::FormFactor formFactor;
+        GBFSVehicleType::PropulsionType propulsion;
+        RentalVehicle::VehicleType type;
+    } const type_map[] = {
+        { GBFSVehicleType::UndefinedFormFactor, GBFSVehicleType::UndefinedPropulsion, RentalVehicle::Unknown },
+        { GBFSVehicleType::Bicycle, GBFSVehicleType::Human, RentalVehicle::Bicycle },
+        { GBFSVehicleType::Bicycle, GBFSVehicleType::ElectricAssist, RentalVehicle::Pedelec },
+        { GBFSVehicleType::Scooter, GBFSVehicleType::Electric, RentalVehicle::ElectricKickScooter },
+        { GBFSVehicleType::Scooter, GBFSVehicleType::ElectricAssist, RentalVehicle::ElectricKickScooter },
+        { GBFSVehicleType::Scooter, GBFSVehicleType::UndefinedPropulsion, RentalVehicle::ElectricKickScooter },
+        { GBFSVehicleType::Moped, GBFSVehicleType::Electric, RentalVehicle::ElectricMoped },
+        { GBFSVehicleType::Moped, GBFSVehicleType::UndefinedPropulsion, RentalVehicle::ElectricMoped },
+        { GBFSVehicleType::Car, GBFSVehicleType::Electric, RentalVehicle::Car },
+        { GBFSVehicleType::Car, GBFSVehicleType::Combustion, RentalVehicle::Car },
+    };
+
+    for (const auto &map : type_map) {
+        if (map.formFactor == vehicle.formFactor && map.propulsion == vehicle.propulsionType) {
+            return map.type;
+        }
+    }
+
+    qDebug() << "unhandled vehicle type:" << vehicle.formFactor << vehicle.propulsionType;
+    return RentalVehicle::Unknown;
+}
+
+// we get some address values just being " , "...
+static QString cleanAddress(const QString &input)
+{
+    if (std::any_of(input.begin(), input.end(), std::mem_fn(qOverload<>(&QChar::isLetter)))) {
+        return input;
+    }
+    return {};
+}
+
 static void appendResults(const GBFSService &service, const LocationRequest &req, QueryContext *context)
 {
     GBFSStore store(service.systemId);
+    GBFSVehicleTypes vehicleTypes(service);
 
     RentalVehicleNetwork network;
     const auto sysInfoDoc = store.loadData(GBFS::SystemInformation);
-    const auto sysInfo = sysInfoDoc.object().value(QLatin1String("data")).toObject();
+    const auto sysInfo = GBFSReader::dataObject(sysInfoDoc);
     network.setName(sysInfo.value(QLatin1String("name")).toString());
 
     const auto stationsDoc = store.loadData(GBFS::StationInformation);
-    const auto stations = stationsDoc.object().value(QLatin1String("data")).toObject().value(QLatin1String("stations")).toArray();
+    const auto stations = GBFSReader::dataValue(stationsDoc, QLatin1String("stations")).toArray();
 
     std::vector<QString> selectedStationIds;
     for (const auto &stationV : stations) {
         const auto station = stationV.toObject();
-        const auto lat = station.value(QLatin1String("lat")).toDouble();
-        const auto lon = station.value(QLatin1String("lon")).toDouble();
-        if (Location::distance(lat, lon, req.latitude(), req.longitude()) > req.maximumDistance()) {
+        const auto lat = GBFSReader::readLatitude(station);
+        const auto lon = GBFSReader::readLongitude(station);
+        if (std::isnan(lat) || std::isnan(lon) || Location::distance(lat, lon, req.latitude(), req.longitude()) > req.maximumDistance()) {
             continue;
         }
         Location loc;
@@ -83,19 +133,26 @@ static void appendResults(const GBFSService &service, const LocationRequest &req
         loc.setName(station.value(QLatin1String("name")).toString());
         const auto stationId = stationIdToString(station.value(QLatin1String("station_id")));
         loc.setIdentifier(service.systemId, stationId);
+        loc.setStreetAddress(cleanAddress(station.value(QLatin1String("address")).toString()));
+        loc.setPostalCode(station.value(QLatin1String("post_code")).toString());
         // TODO cover more properties
 
         RentalVehicleStation s;
         s.setNetwork(network);
         s.setCapacity(station.value(QLatin1String("capacity")).toInt(-1));
-        loc.setData(s);
+        const auto vehicleCapacities = station.value(QLatin1String("vehicle_capacity")).toObject();
+        for (auto it = vehicleCapacities.begin(); it != vehicleCapacities.end(); ++it) {
+            const auto type = gbfs2kptVehicleType(vehicleTypes.vehicleType(it.key()));
+            s.setCapacity(type, it.value().toInt(-1));
+        }
 
+        loc.setData(s);
         selectedStationIds.push_back(stationId);
         context->result.push_back(loc);
     }
 
     const auto statusDoc = store.loadData(GBFS::StationStatus);
-    const auto status = statusDoc.object().value(QLatin1String("data")).toObject().value(QLatin1String("stations")).toArray();
+    const auto status = GBFSReader::dataValue(statusDoc, QLatin1String("stations")).toArray();
     for (const auto &statV : status) {
         const auto stat = statV.toObject();
         const auto id = stationIdToString(stat.value(QLatin1String("station_id")));
@@ -106,20 +163,28 @@ static void appendResults(const GBFSService &service, const LocationRequest &req
 
         auto &loc = context->result[context->result.size() - selectedStationIds.size() + std::distance(selectedStationIds.begin(), it)];
         auto s = loc.rentalVehicleStation();
+
         s.setAvailableVehicles(stat.value(QLatin1String("num_bikes_available")).toInt(-1));
+        const auto availableVehicleTypes = stat.value(QLatin1String("vehicle_types_available")).toArray();
+        for (const auto &v : availableVehicleTypes) {
+            const auto obj = v.toObject();
+            const auto type = gbfs2kptVehicleType(vehicleTypes.vehicleType(obj.value(QLatin1String("vehicle_type_id")).toString()));
+            s.setAvailableVehicles(type, obj.value(QLatin1String("count")).toInt(-1));
+        }
+
         loc.setData(s);
     }
 
     const auto floatingDoc = store.loadData(GBFS::FreeBikeStatus);
-    const auto floating = floatingDoc.object().value(QLatin1String("data")).toObject().value(QLatin1String("bikes")).toArray();
+    const auto floating = GBFSReader::dataValue(floatingDoc, QLatin1String("bikes")).toArray();
     for (const auto &bikeV : floating) {
         const auto bike = bikeV.toObject();
         if (bike.value(QLatin1String("is_reserved")).toBool() || bike.value(QLatin1String("is_disabled")).toBool()) {
             continue;
         }
-        const auto lat = bike.value(QLatin1String("lat")).toDouble();
-        const auto lon = bike.value(QLatin1String("lon")).toDouble();
-        if (Location::distance(lat, lon, req.latitude(), req.longitude()) > req.maximumDistance()) {
+        const auto lat = GBFSReader::readLatitude(bike);
+        const auto lon = GBFSReader::readLongitude(bike);
+        if (std::isnan(lat) || std::isnan(lon) || Location::distance(lat, lon, req.latitude(), req.longitude()) > req.maximumDistance()) {
             continue;
         }
 
@@ -130,11 +195,23 @@ static void appendResults(const GBFSService &service, const LocationRequest &req
         const auto stationId = bike.value(QLatin1String("bike_id")).toString();
         loc.setIdentifier(service.systemId, stationId);
 
-        // TODO vehicle type, remaining range, deep rental links
+        // TODO deep rental links
         RentalVehicle vehicle;
         vehicle.setNetwork(network);
-        loc.setData(vehicle);
 
+        auto vehicleTypeId = bike.value(QLatin1String("vehicle_type_id")).toString();
+        if (vehicleTypeId.isEmpty()) { // non-compliant format used eg. by Lime
+            vehicleTypeId = bike.value(QLatin1String("vehicle_type")).toString();
+        }
+        const auto vehicleType = vehicleTypes.vehicleType(vehicleTypeId);
+        vehicle.setType(gbfs2kptVehicleType(vehicleType));
+
+        const auto range = bike.value(QLatin1String("current_range_meters")).toInt();
+        if (range > 0) { // there's too many reporting 0 for unknown that we can assume 0 means actually empty...
+            vehicle.setRemainingRange(range);
+        }
+
+        loc.setData(vehicle);
         context->result.push_back(loc);
     }
 
@@ -179,6 +256,7 @@ bool GBFSBackend::queryLocation(const LocationRequest &req, LocationReply *reply
 
         context->pendingJobs++;
         auto updateJob = new GBFSJob(nam, reply);
+        updateJob->setRequestedData({GBFS::StationInformation, GBFS::StationStatus, GBFS::FreeBikeStatus, GBFS::VehicleTypes});
         QObject::connect(updateJob, &GBFSJob::finished, reply, [this, context, reply, updateJob, req]() {
             context->pendingJobs--;
             updateJob->deleteLater();
@@ -190,7 +268,7 @@ bool GBFSBackend::queryLocation(const LocationRequest &req, LocationReply *reply
             }
 
             if (context->pendingJobs == 0 &&  !context->stillStarting) {
-                if (context->hasError) {
+                if (context->hasError && context->result.empty()) {
                     addError(reply, Reply::NetworkError, context->errorMessage);
                 } else {
                     addResult(reply, std::move(context->result));
